package com.loafle.commons.server.socket.netty.handler.codec;

import io.netty.buffer.Unpooled;
import io.netty.channel.ChannelFutureListener;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandler;
import io.netty.channel.ChannelPipeline;

import java.util.List;

/**
 * This handler does all the heavy lifting for you to run a websocket server.
 *
 * It takes care of websocket handshaking as well as processing of control frames (Close, Ping, Pong). Text and Binary
 * data frames are passed to the next handler in the pipeline (implemented by you) for processing.
 *
 * See <tt>io.netty.example.http.websocketx.html5.WebSocketServer</tt> for usage.
 *
 * The implementation of this handler assumes that you just want to run  a websocket server and not process other types
 * HTTP requests (like GET and POST). If you wish to support both HTTP requests and websockets in the one server, refer
 * to the <tt>io.netty.example.http.websocketx.server.WebSocketServer</tt> example.
 *
 * To know once a handshake was done you can intercept the
 * {@link ChannelInboundHandler#userEventTriggered(ChannelHandlerContext, Object)} and check if the event was instance
 * of {@link HandshakeComplete}, the event will contain extra information about the handshake such as the request and
 * selected subprotocol.
 */
public class WebSocketServerProtocolHandler extends WebSocketProtocolHandler {

  private final boolean allowExtensions;
  private final int maxFramePayloadLength;
  private final boolean allowMaskMismatch;

  public WebSocketServerProtocolHandler(boolean allowExtensions) {
    this(allowExtensions, 65536);
  }

  public WebSocketServerProtocolHandler(boolean allowExtensions, int maxFrameSize) {
    this(allowExtensions, maxFrameSize, false);
  }

  public WebSocketServerProtocolHandler(boolean allowExtensions, int maxFrameSize, boolean allowMaskMismatch) {
    this.allowExtensions = allowExtensions;
    maxFramePayloadLength = maxFrameSize;
    this.allowMaskMismatch = allowMaskMismatch;
  }

  @Override
  public void handlerAdded(ChannelHandlerContext ctx) {
    ChannelPipeline cp = ctx.pipeline();

    WebSocketFrameDecoder decoder = new WebSocket13FrameDecoder(true, allowExtensions, maxFramePayloadLength,
        allowMaskMismatch);
    WebSocketFrameEncoder encoder = new WebSocket13FrameEncoder(false);

    cp.addBefore(ctx.name(), WebSocketFrameDecoder.class.getName(), decoder);
    cp.addBefore(ctx.name(), WebSocketFrameEncoder.class.getName(), encoder);

    if (cp.get(Utf8FrameValidator.class) == null) {
      // Add the UFT8 checking before this one.
      ctx.pipeline().addBefore(ctx.name(), Utf8FrameValidator.class.getName(), new Utf8FrameValidator());
    }
  }

  @Override
  protected void decode(ChannelHandlerContext ctx, WebSocketFrame frame, List<Object> out) throws Exception {
    if (frame instanceof CloseWebSocketFrame) {
      ctx.writeAndFlush(Unpooled.EMPTY_BUFFER).addListener(ChannelFutureListener.CLOSE);
      return;
    }
    super.decode(ctx, frame, out);
  }

  @Override
  public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
    ctx.fireExceptionCaught(cause);
    ctx.close();
  }
}